@@ -1,5 +1,5 @@
 from django.conf import settings
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.utils.translation import ugettext as _
 
 from crispy_forms.helper import FormHelper
@@ -8,8 +8,9 @@ from crispy_forms.layout import Hidden, Submit
 
 class RegistrationFormHelper(FormHelper):
     form_action = reverse('registration_register')
+    include_media = False
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, request, *args, **kwargs):
         super(RegistrationFormHelper, self).__init__(*args, **kwargs)
         self.add_input(Submit('submit', _('Sign up')))
 
