@@ -11,7 +11,7 @@ urlpatterns = [
     url(r'^talks/', include('wafer.talks.urls')),
     url(r'^sponsors/', include('wafer.sponsors.urls')),
     url(r'^pages/', include('wafer.pages.urls')),
-    url(r'^admin/', include(admin.site.urls)),
+    url(r'^admin/', admin.site.urls),
     url(r'^markitup/', include('markitup.urls')),
     url(r'^schedule/', include('wafer.schedule.urls')),
     url(r'^tickets/', include('wafer.tickets.urls')),
